@@ -1,4 +1,4 @@
-/* $Id: minimum.cpp 1561 2010-06-13 13:08:03Z stefan $ */
+/* $Id: minimum.cpp 1649 2010-12-19 17:25:22Z stefan $ */
 // Copyright (C) 2002, International Business Machines
 // Corporation and others.  All Rights Reserved.
 
@@ -9,7 +9,11 @@ int main(int argc, const char *argv[])
      int status;
      if (argc < 2) {
 #if defined(SAMPLEDIR)
-          status = model.readMps(SAMPLEDIR "/p0033.mps", true);
+/*
+  SAMPLEDIR should "path/to/Data/Sample/"
+  Include the quotes and final path separator.
+*/
+          status = model.readMps(SAMPLEDIR "p0033.mps", true);
 #else
           fprintf(stderr, "Do not know where to find sample MPS files.\n");
           exit(1);
